//+------------------------------------------------------------------+
//|                                          Guppy MMA oscilator.mq4 |
//|                                                           mladen |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_buffers 2
#property indicator_color1  Green
#property indicator_color2  Red

//
//
//
//
//

extern string TimeFrame             = "current time frame";
extern int    Price                 = PRICE_CLOSE;
extern int    SignalPeriod          = 9;
extern bool   ShowArrowsOnZeroCross = false;

//
//
//
//
//

extern bool  alertsOn        = false;
extern bool  alertsOnCurrent = true;
extern bool  alertsMessage   = true;
extern bool  alertsSound     = false;
extern bool  alertsEmail     = false;

//
//
//
//
//

double upBuffer[];
double dnBuffer[];
double buffer1[];
double buffer2[];
double trend[];
double periods[]={3,5,8,10,12,15,30,35,40,45,50,60};
int    persize;

//
//
//
//
//

string indicatorFileName;
bool   returnBars;
bool   calculateValue;
int    timeFrame;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   IndicatorBuffers(5);
   persize         = ArraySize(periods);
   calculateValue  = (TimeFrame=="calculateValue");
      if (calculateValue)
      {
         SetIndexBuffer(0,buffer1);
         SetIndexBuffer(1,buffer2);
         SetIndexBuffer(2,upBuffer);
         SetIndexBuffer(3,dnBuffer);
         return(0);
      }      
      else      
      {
         SetIndexBuffer(0,upBuffer);  SetIndexStyle(0,DRAW_ARROW); SetIndexArrow(0,108);
         SetIndexBuffer(1,dnBuffer);  SetIndexStyle(1,DRAW_ARROW); SetIndexArrow(1,108);
         SetIndexBuffer(2,buffer1);
         SetIndexBuffer(3,buffer2);
      }
      SetIndexBuffer(4,trend);
   
   //
   //
   //
   //
   //
   
   indicatorFileName = WindowExpertName();
   returnBars        = (TimeFrame=="returnBars");     if (returnBars) return(0);
   timeFrame         = stringToTimeFrame(TimeFrame);
      IndicatorShortName("GMMA oscialltor "+timeFrameToString(timeFrame));
   return(0);
}
int deinit() { return(0); }

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
      limit=MathMin(Bars-counted_bars,Bars-1);
      if (returnBars)  { buffer1[0] = limit+1; return(0); }
      
   //
   //
   //
   //
   //

   if (calculateValue || timeFrame==Period())
   {
      double alpha = 2.0/(1.0+SignalPeriod);
      for(i=limit; i>=0; i--)
      {
            double sum  = 0;
            for(int j=0; j<persize; j++)
            {
               if (periods[j]<30)
                     sum  += iMA(NULL,0,periods[j],0,MODE_EMA,Price,i);
               else  sum  -= iMA(NULL,0,periods[j],0,MODE_EMA,Price,i);
            }
            buffer1[i] = sum*10.0;
            buffer2[i] = buffer2[i+1]+alpha*(buffer1[i]-buffer2[i+1]);
            if (calculateValue) continue;
      
         //
         //
         //
         //
         //
      
            upBuffer[i]  = EMPTY_VALUE;
            dnBuffer[i]  = EMPTY_VALUE;
            trend[i]     = trend[i+1];
            if (ShowArrowsOnZeroCross)
               {
                  if (buffer1[i]>0) trend[i]= 1;
                  if (buffer1[i]<0) trend[i]=-1;
               }
            else                     
               {
                  if (buffer1[i]>buffer2[i]) trend[i]= 1;
                  if (buffer1[i]<buffer2[i]) trend[i]=-1;
               }
            if (trend[i] != trend[i+1])
               {
                  if (trend[i] ==  1) upBuffer[i] = Low[i]-iATR(NULL,0,20,i);
                  if (trend[i] == -1) dnBuffer[i] = High[i]+iATR(NULL,0,20,i);
               }            
      }
      if (!calculateValue) manageAlerts();
      return(0);
   }

   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars-1,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
   for(i=limit; i>=0; i--)
   {
      int y = iBarShift(NULL,timeFrame,Time[i]);
         buffer1[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Price,SignalPeriod,0,y);
         buffer2[i]  = iCustom(NULL,timeFrame,indicatorFileName,"calculateValue",Price,SignalPeriod,1,y);
         upBuffer[i] = EMPTY_VALUE;
         dnBuffer[i] = EMPTY_VALUE;
         trend[i]    = trend[i+1];

         if (ShowArrowsOnZeroCross)
            {
               if (buffer1[i]>0) trend[i]= 1;
               if (buffer1[i]<0) trend[i]=-1;
            }
         else                     
            {
               if (buffer1[i]>buffer2[i]) trend[i]= 1;
               if (buffer1[i]<buffer2[i]) trend[i]=-1;
            }
         if (trend[i] != trend[i+1])
            {
               if (trend[i] ==  1) upBuffer[i] = Low[i] -iATR(NULL,0,20,i);
               if (trend[i] == -1) dnBuffer[i] = High[i]+iATR(NULL,0,20,i);
            }            
   }
   manageAlerts();
     
   //
   //
   //
   //
   //
   
   return(0);
}

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+  
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1; whichBar = iBarShift(NULL,0,iTime(NULL,timeFrame,whichBar));
      if (trend[whichBar] != trend[whichBar+1])
      if (trend[whichBar] == 1)
            doAlert(whichBar,"up");
      else  doAlert(whichBar,"down");
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[forBar]) {
          previousAlert  = doWhat;
          previousTime   = Time[forBar];

          //
          //
          //
          //
          //

          message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," GMMA trend changed to ",doWhat);
             if (alertsMessage) Alert(message);
             if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"GMMA "),message);
             if (alertsSound)   PlaySound("alert2.wav");
      }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = StringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string StringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}